@@ -439,7 +439,6 @@ static int parse_adaptation_sets(AVFormatContext *s)
             continue;
         else if (state == new_set && !strncmp(p, "id=", 3)) {
             void *mem = av_realloc(w->as, sizeof(*w->as) * (w->nb_as + 1));
-            const char *comma;
             if (mem == NULL)
                 return AVERROR(ENOMEM);
             w->as = mem;
@@ -448,11 +447,6 @@ static int parse_adaptation_sets(AVFormatContext *s)
             w->as[w->nb_as - 1].streams = NULL;
             p += 3; // consume "id="
             q = w->as[w->nb_as - 1].id;
-            comma = strchr(p, ',');
-            if (!comma || comma - p >= sizeof(w->as[w->nb_as - 1].id)) {
-                av_log(s, AV_LOG_ERROR, "'id' in 'adaptation_sets' is malformed.\n");
-                return AVERROR(EINVAL);
-            }
             while (*p != ',') *q++ = *p++;
             *q = 0;
             p++;
@@ -462,12 +456,11 @@ static int parse_adaptation_sets(AVFormatContext *s)
             state = parsing_streams;
         } else if (state == parsing_streams) {
             struct AdaptationSet *as = &w->as[w->nb_as - 1];
-            int ret = av_reallocp_array(&as->streams, ++as->nb_streams,
-                                        sizeof(*as->streams));
-            if (ret < 0)
-                return ret;
             q = p;
             while (*q != '\0' && *q != ',' && *q != ' ') q++;
+            as->streams = av_realloc(as->streams, sizeof(*as->streams) * ++as->nb_streams);
+            if (as->streams == NULL)
+                return AVERROR(ENOMEM);
             as->streams[as->nb_streams - 1] = to_integer(p, q - p + 1);
             if (as->streams[as->nb_streams - 1] < 0 ||
                 as->streams[as->nb_streams - 1] >= s->nb_streams) {
@@ -490,14 +483,6 @@ static int webm_dash_manifest_write_header(AVFormatContext *s)
     double start = 0.0;
     int ret;
     WebMDashMuxContext *w = s->priv_data;
-
-    for (unsigned i = 0; i < s->nb_streams; i++) {
-        enum AVCodecID codec_id = s->streams[i]->codecpar->codec_id;
-        if (codec_id != AV_CODEC_ID_VP8    && codec_id != AV_CODEC_ID_VP9 &&
-            codec_id != AV_CODEC_ID_VORBIS && codec_id != AV_CODEC_ID_OPUS)
-            return AVERROR(EINVAL);
-    }
-
     ret = parse_adaptation_sets(s);
     if (ret < 0) {
         goto fail;
